pub struct OperationLog {
db: Database,
oldest_entry: Arc<RwLock<Option<ChronologicalOperationLogKey>>>,
}Fields
db: Databaseoldest_entry: Arc<RwLock<Option<ChronologicalOperationLogKey>>>Implementations
Sourceimpl OperationLog
impl OperationLog
pub fn new(db: Database) -> Self
Sourceasync fn get_oldest_operation_log_key(
&self,
) -> Option<ChronologicalOperationLogKey>
async fn get_oldest_operation_log_key( &self, ) -> Option<ChronologicalOperationLogKey>
Will return the oldest operation log key in the database and cache the result. If no entry exists yet the DB will be queried on each call till an entry is present.
pub async fn add_operation_log_entry_dbtx( &self, dbtx: &mut DatabaseTransaction<'_>, operation_id: OperationId, operation_type: &str, operation_meta: impl Serialize, )
pub async fn add_operation_log_entry_dbtx_with_creation_time( &self, dbtx: &mut DatabaseTransaction<'_>, operation_id: OperationId, operation_type: &str, operation_meta: impl Serialize, creation_time: SystemTime, )
pub async fn list_operations( &self, limit: usize, last_seen: Option<ChronologicalOperationLogKey>, ) -> Vec<(ChronologicalOperationLogKey, OperationLogEntry)>
paginate_operations_rev insteadSourcepub async fn paginate_operations_rev(
&self,
limit: usize,
last_seen: Option<ChronologicalOperationLogKey>,
) -> Vec<(ChronologicalOperationLogKey, OperationLogEntry)>
pub async fn paginate_operations_rev( &self, limit: usize, last_seen: Option<ChronologicalOperationLogKey>, ) -> Vec<(ChronologicalOperationLogKey, OperationLogEntry)>
Returns the last limit operations. To fetch the next page, pass the
last operation’s ChronologicalOperationLogKey as start_after.
pub async fn get_operation( &self, operation_id: OperationId, ) -> Option<OperationLogEntry>
pub async fn get_operation_dbtx( dbtx: &mut DatabaseTransaction<'_>, operation_id: OperationId, ) -> Option<OperationLogEntry>
Sourcepub async fn set_operation_outcome(
db: &Database,
operation_id: OperationId,
outcome: &(impl Serialize + Debug),
) -> Result<()>
pub async fn set_operation_outcome( db: &Database, operation_id: OperationId, outcome: &(impl Serialize + Debug), ) -> Result<()>
Sets the outcome of an operation
Sourcepub fn outcome_or_updates<U, S>(
db: &Database,
operation_id: OperationId,
operation_log_entry: &OperationLogEntry,
is_terminal: impl Fn(&U) -> bool + MaybeSend + MaybeSync + 'static,
stream_gen: impl FnOnce() -> S,
) -> UpdateStreamOrOutcome<U>
pub fn outcome_or_updates<U, S>( db: &Database, operation_id: OperationId, operation_log_entry: &OperationLogEntry, is_terminal: impl Fn(&U) -> bool + MaybeSend + MaybeSync + 'static, stream_gen: impl FnOnce() -> S, ) -> UpdateStreamOrOutcome<U>
Returns an a UpdateStreamOrOutcome enum that can be converted into
an update stream for easier handling using
UpdateStreamOrOutcome::into_stream but can also be matched over to
shortcut the handling of final outcomes.
is_terminal reports whether a given update is a final state of the
operation (no further update will ever follow it). Only terminal
updates are cached as the operation’s outcome — an update stream that
ends early (a consumer-side error path) must not persist a
non-terminal update as the durable outcome, since later subscribers
would then be short-circuited to it forever. The same predicate also
validates outcomes READ from the cache: a cached outcome that no
longer deserializes (e.g. written by an older version with a different
update type) or that is non-terminal (frozen by a previous version
that cached whatever update a stream ended on) is discarded with a
warning and the outcome is rebuilt from the update stream — whose
terminal end then overwrites the stale cache — instead of
short-circuiting subscribers to it or panicking.
Sourcepub async fn optimistically_set_operation_outcome(
db: &Database,
operation_id: OperationId,
outcome: &(impl Serialize + Debug),
)
pub async fn optimistically_set_operation_outcome( db: &Database, operation_id: OperationId, outcome: &(impl Serialize + Debug), )
Tries to set the outcome of an operation, but only logs an error if it fails and does not return it. Since the outcome can always be recomputed from an update stream, failing to save it isn’t a problem in cases where we do this merely for caching.